&ANALYZE-SUSPEND _VERSION-NUMBER AB_v10r12 GUI ADM2
&ANALYZE-RESUME
&Scoped-define WINDOW-NAME CURRENT-WINDOW
&ANALYZE-SUSPEND _UIB-CODE-BLOCK _CUSTOM _DEFINITIONS fFrameWin 
/*------------------------------------------------------------------------

  File: sdbtreeview.w

  Description: SmartObject wrapper for treeview control from Microsoft
               This tree can act effectively as a SmartDataBrowser but
               with a tree view of the records. e.g. Area tree, users tree etc
               
               Currently uses MS TreeView V6.0

  Notes: Use this control to show a tree and respond to events from it
  
         The tree is created by calling the Add internal procedure to add
         nodes to the tree control.

         You can check which node is selected at any time by querying the
         attribute 'AcSelectedItemKey' which returns the key of the selected
         node.
         
         You can respond to the user double clicking on a leaf node by 
         subscribing to the event 'TreeDblClickLeaf'. The internal procedure
         fired must take a single char parameter which will be the key of
         the node clicked. This subscribe can be done easily by adding a
         SmartLink of type 'TreeDblClickLeaf' from this SmartObject to 
         another SmartObject and coding a procedure within the taret object
         called 'TreeDblClickLeaf' (with the single char input parameter).

         SmartDataBrowserTree functionality
         ----------------------------------
        
        The tree in this instance has a data source of fsurvdo0.w.
        This provides the survey record level of the view, which is 
        the first 'level'. For each survey, it gets all the levels 
        for the survey from flevldo0.w in BuildTreeTable.And build
        accordingly
        The container tells the tree which fields are responsible 
        for what by setting functions and then retrieving them 
        in the build.
           
           Survey Attributes
           {set AcSurveyCode 'fsurvey_code' h_ftreefr0}.
           {set AcSurveyDesc  'dsc' h_ftreefr0}.
           {set AcSurveyActive  'active' h_ftreefr0}. 
           
           Level Attributes
           {set AcIdFieldName 'flevel_code' h_ftreefr0}.
           {set AcparentIdFieldName 'parent_flevel_code' h_ftreefr0}.
           {set AcTextFieldName 'dsc' h_ftreefr0}.
           {set AcLevelActive 'activelevel' h_ftreefr0}. 
           {set AcLevelLeaf    'leaf' h_ftreefr0}.

     
         
         These settings are to tell the tree the field names for
         - The 'id' field name i.e. The field uniquely identifying the record
           and also the field which relates to the reflexive relationship on
           the table (i.e. it joins to AcparentIdFieldName
         - The 'parentId' field name, which joins to id as discussed
         - The 'text' field name which is used to populate the text on each node
         
         NOTE: This object must be initialised after the SDO object to which
               it is linked (i.e. must be lower on the frame of the container)
         
        
         Still Outstanding
         -----------------
         - Single clicking (TreeClick event/link)
         - Context menu (currently some code commented out in initializeObject
           and OCX.MouseDown. Have to use an API call to show the context menu
           but apart from that looked OK - had to left click then right click
           to select a node then get the context menu for it which looked a
           bit strange). Probably best to let the developer generate a context 
           menu in their code then assign the FRAME {&FRAME-NAME}:MENU to this
           menu handle.
 
    
 
 
------------------------------------------------------------------------*/
/*          This .W file was created with the Progress AppBuilder.      */
/*----------------------------------------------------------------------*/

/* Create an unnamed pool to store all the widgets created 
     by this procedure. This is a good default which assures
     that this procedure's triggers and internal procedures 
     will execute in this procedure's storage, and that proper
     cleanup will occur on deletion of the procedure. */

CREATE WIDGET-POOL.

/* ***************************  Definitions  ************************** */

/* Parameters Definitions ---                                           */

DEFINE NEW GLOBAL SHARED VARIABLE GoisDebug                 AS CHAR             NO-UNDO.
DEFINE NEW GLOBAL SHARED VARIABLE GhDebugWindowProc         AS HANDLE           NO-UNDO.

/* Local Variable Definitions ---                                       */
DEFINE VARIABLE l-cUIBMode      AS CHAR                 NO-UNDO.

/* Images for nodes default to 1 and 2 - can change with {set} on AiImage/AiSelectedImage */
DEFINE VARIABLE AiImage              AS INT INIT 1      NO-UNDO.
DEFINE VARIABLE AiSelectedImage      AS INT INIT 2      NO-UNDO.
DEFINE VARIABLE AiLeafImage          AS INT INIT 3      NO-UNDO.
DEFINE VARIABLE AiLeafSelImage       AS INT INIT 4      NO-UNDO.
DEFINE VARIABLE AlSecurityCheck      AS LOGICAL         NO-UNDO.

DEFINE VARIABLE AlTreeInitialized    AS LOG    INIT FALSE  NO-UNDO.

/* Signal whether we're in 'SmartDataBrowse' mode i.e. with a data source
   object which we 'browse' using the tree */
DEFINE VARIABLE LlSDBMode              AS LOG INITIAL FALSE NO-UNDO.
DEFINE VARIABLE AcIdFieldName          AS CHAR              NO-UNDO.
DEFINE VARIABLE AcparentIdFieldName    AS CHAR              NO-UNDO.
DEFINE VARIABLE AcTextFieldName        AS CHAR              NO-UNDO.
DEFINE VARIABLE AcAutoExpandSingleRoot AS CHAR              NO-UNDO.
DEFINE VARIABLE AlAutoExpandAllNodes   AS LOG INIT no       NO-UNDO.
DEFINE VARIABLE AlAutoSortAllNodes     AS LOG INIT YES      NO-UNDO.

DEFINE VARIABLE AcSurveyCode           AS CHAR              NO-UNDO.
DEFINE VARIABLE AcSurveyDesc           AS CHARACTER         NO-UNDO.
DEFINE VARIABLE AcSurveyActive         AS CHAR              NO-UNDO.
DEFINE VARIABLE AcLevelActive          AS CHAR              NO-UNDO.
DEFINE VARIABLE AcLevelLeaf            AS CHARACTER         NO-UNDO.
DEFINE VARIABLE LhLevelData            AS HANDLE            NO-UNDO.
DEFINE VARIABLE AcLevelHandle          AS HANDLE            NO-UNDO.

DEFINE VARIABLE LcIdList               AS CHARACTER         NO-UNDO.
DEFINE VARIABLE LhContainerSource      AS HANDLE            NO-UNDO.
DEFINE VARIABLE LcCurrentSet           AS CHARACTER         NO-UNDO.
DEFINE VARIABLE LcOldSet               AS CHARACTER         NO-UNDO.
DEFINE VARIABLE LcNodeKey              AS CHARACTER         NO-UNDO.
DEFINE VARIABLE LcLevelCode            AS CHARACTER         NO-UNDO.
DEFINE VARIABLE LcTreeMode             AS CHARACTER         NO-UNDO.

/* Unlinked root is allowed when acting as a SmartDataBrowser. The attribute
   allows the container to specify that a root node should be created which
   will parent all other (record) nodes, but will not be linked to a record
   itself. The data-source must handle this in fetchRowIdent, to which a
   RowIdent of 'UnlinkedRoot' will be passed.
*/
DEFINE VARIABLE AlIncludeUnlinkedRoot  AS LOG INITIAL FALSE NO-UNDO. 
DEFINE VARIABLE AlIncludeUnlinkedRootText AS CHAR INITIAL "Surveys for Set" NO-UNDO.

DEFINE VARIABLE AlAutoSelectRootNodeOnOpen  AS LOG INIT yes  NO-UNDO.

DEFINE VARIABLE LcCurrentRowIdent           AS CHAR             NO-UNDO.
DEFINE VARIABLE LlTransactionCompleted      AS LOG INIT FALSE   NO-UNDO.
DEFINE VARIABLE LcUserSecurityProfileList   AS CHARACTER        NO-UNDO.

DEFINE TEMP-TABLE ttNode
    FIELD id            AS CHAR
    FIELD parentId      AS CHAR
    FIELD nodeText      AS CHAR
    FIELD RowIdent      AS CHAR
    FIELD SecurityLevel AS CHAR
    FIELD ImageName     AS CHAR
    FIELD active        AS CHAR
    FIELD usercode      AS CHAR
    FIELD InstanceSeqNo AS INT
    
    /* For Progress Tree */
    FIELD nodeTextHandle AS HANDLE
    FIELD nodeOpenerImageHandle AS HANDLE
    FIELD nodeFrameHandle AS HANDLE

    /*
    INDEX parentIdSeqNo parentId InstanceSeqNo usercode
    INDEX parentId parentId usercode
    */
    INDEX id id
    /*INDEX rowIdent rowIdent*/
    .

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME


&ANALYZE-SUSPEND _UIB-PREPROCESSOR-BLOCK 

/* ********************  Preprocessor Definitions  ******************** */

&Scoped-define PROCEDURE-TYPE SmartFrame
&Scoped-define DB-AWARE no

&Scoped-define ADM-CONTAINER FRAME

&Scoped-define ADM-SUPPORTED-LINKS Data-Target,Data-Source,Page-Target

/* Name of designated FRAME-NAME and/or first browse and/or first query */
&Scoped-define FRAME-NAME fTreeViewParent

/* Custom List Definitions                                              */
/* List-1,List-2,List-3,List-4,List-5,List-6                            */

/* _UIB-PREPROCESSOR-BLOCK-END */
&ANALYZE-RESUME


/* ************************  Function Prototypes ********************** */

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD FcGetRowToPositionAfterDelete fFrameWin 
FUNCTION FcGetRowToPositionAfterDelete RETURNS CHARACTER
  ( /* parameter-definitions */ )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD FcNodeImageName fFrameWin 
FUNCTION FcNodeImageName RETURNS CHARACTER
  ( BUFFER ttNode FOR ttNode )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD FlIsNodeExpanded fFrameWin 
FUNCTION FlIsNodeExpanded RETURNS LOGICAL
  ( INPUT IcNodeKey         AS CHAR )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD GetAcSelectedItemKey fFrameWin 
FUNCTION GetAcSelectedItemKey RETURNS CHARACTER
  ( /* parameter-definitions */ )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD getTreeInitialized fFrameWin 
FUNCTION getTreeInitialized RETURNS CHARACTER
  ( /* parameter-definitions */ )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD Key2ComHandle fFrameWin 
FUNCTION Key2ComHandle RETURNS COM-HANDLE
  ( INPUT i-cKey AS CHAR )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD SetAcAutoExpandSingleRoot fFrameWin 
FUNCTION SetAcAutoExpandSingleRoot RETURNS CHARACTER
  ( INPUT i AS CHAR /* parameter-definitions */ )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD SetAcIdFieldName fFrameWin 
FUNCTION SetAcIdFieldName RETURNS CHARACTER
  ( INPUT i AS CHAR )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD setAcImageListToUse fFrameWin 
FUNCTION setAcImageListToUse RETURNS CHARACTER
  ( INPUT Ic AS CHAR )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD SetAcLevelActive fFrameWin 
FUNCTION SetAcLevelActive RETURNS CHARACTER
  ( INPUT i AS CHAR )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD SetAcLevelHandle fFrameWin 
FUNCTION SetAcLevelHandle RETURNS HANDLE
  ( INPUT i AS HANDLE )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD SetAcLevelLeaf fFrameWin 
FUNCTION SetAcLevelLeaf RETURNS CHARACTER
  ( INPUT i AS CHAR )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD SetAcparentIdFieldName fFrameWin 
FUNCTION SetAcparentIdFieldName RETURNS CHARACTER
  ( INPUT i AS CHAR )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD SetAcSelectedNode fFrameWin 
FUNCTION SetAcSelectedNode RETURNS CHARACTER
  ( INPUT cRowIdent AS CHAR )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD SetAcSurveyActive fFrameWin 
FUNCTION SetAcSurveyActive RETURNS CHAR
  ( INPUT i AS CHAR )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD SetAcSurveyCode fFrameWin 
FUNCTION SetAcSurveyCode RETURNS CHARACTER
  ( INPUT i AS CHAR)  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD SetAcSurveyDesc fFrameWin 
FUNCTION SetAcSurveyDesc RETURNS CHARACTER
  ( INPUT i AS CHAR )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD SetAcTextFieldName fFrameWin 
FUNCTION SetAcTextFieldName RETURNS CHARACTER
  ( INPUT i AS CHAR )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD SetAcTreeMode fFrameWin 
FUNCTION SetAcTreeMode RETURNS CHARACTER
  ( INPUT i AS CHAR )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD SetAiImage fFrameWin 
FUNCTION SetAiImage RETURNS CHARACTER
  ( INPUT val AS CHAR /* parameter-definitions */ )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD setAiLeafImage fFrameWin 
FUNCTION setAiLeafImage RETURNS CHARACTER
  ( INPUT Ii AS INT )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD setAiLeafSelImage fFrameWin 
FUNCTION setAiLeafSelImage RETURNS CHARACTER
  ( INPUT Ii AS INT )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD setAiSelectedImage fFrameWin 
FUNCTION setAiSelectedImage RETURNS CHARACTER
  ( INPUT Ii AS INT )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD setAlAutoExpandAllNodes fFrameWin 
FUNCTION setAlAutoExpandAllNodes RETURNS CHARACTER
  ( INPUT Il AS LOG )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD setAlAutoSelectRootNodeOnOpen fFrameWin 
FUNCTION setAlAutoSelectRootNodeOnOpen RETURNS CHARACTER
  ( INPUT Il AS LOG )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD setAlAutoSortAllNodes fFrameWin 
FUNCTION setAlAutoSortAllNodes RETURNS CHARACTER
  ( INPUT Il AS LOG /* parameter-definitions */ )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD setAlIncludeUnlinkedRoot fFrameWin 
FUNCTION setAlIncludeUnlinkedRoot RETURNS CHARACTER
  ( INPUT Il AS LOG )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD setAlIncludeUnlinkedRootText fFrameWin 
FUNCTION setAlIncludeUnlinkedRootText RETURNS CHARACTER
  ( INPUT Ic AS CHAR )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD setAlSecurityCheck fFrameWin 
FUNCTION setAlSecurityCheck RETURNS CHARACTER
  ( INPUT Il    AS LOGICAL )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION-FORWARD SetAlTreeInitialized fFrameWin 
FUNCTION SetAlTreeInitialized RETURNS CHARACTER
  ( INPUT i AS CHAR )  FORWARD.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME


/* ***********************  Control Definitions  ********************** */


/* ************************  Frame Definitions  *********************** */

DEFINE FRAME fTreeViewParent
    WITH 1 DOWN KEEP-TAB-ORDER OVERLAY 
         SIDE-LABELS NO-UNDERLINE THREE-D 
         AT COL 1 ROW 1
         SIZE 39.72 BY 11.27
         BGCOLOR 15 FONT 8.


/* *********************** Procedure Settings ************************ */

&ANALYZE-SUSPEND _PROCEDURE-SETTINGS
/* Settings for THIS-PROCEDURE
   Type: SmartFrame
   Allow: Basic,Browse,DB-Fields,Query,Smart
   Container Links: Data-Target,Data-Source,Page-Target
   Other Settings: PERSISTENT-ONLY
 */

/* This procedure should always be RUN PERSISTENT.  Report the error,  */
/* then cleanup and return.                                            */
IF NOT THIS-PROCEDURE:PERSISTENT THEN DO:
  MESSAGE "{&FILE-NAME} should only be RUN PERSISTENT.":U
          VIEW-AS ALERT-BOX ERROR BUTTONS OK.
  RETURN.
END.

&ANALYZE-RESUME _END-PROCEDURE-SETTINGS

/* *************************  Create Window  ************************** */

&ANALYZE-SUSPEND _CREATE-WINDOW
/* DESIGN Window definition (used by the UIB) 
  CREATE WINDOW fFrameWin ASSIGN
         HEIGHT             = 11.96
         WIDTH              = 42.
/* END WINDOW DEFINITION */
                                                                        */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _CUSTOM _INCLUDED-LIB fFrameWin 
/* ************************* Included-Libraries *********************** */

{acontml0.i}
{atreeml0.i}

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME




/* ***********  Runtime Attributes and AppBuilder Settings  *********** */

&ANALYZE-SUSPEND _RUN-TIME-ATTRIBUTES
/* SETTINGS FOR WINDOW fFrameWin
  VISIBLE,,RUN-PERSISTENT                                               */
/* SETTINGS FOR FRAME fTreeViewParent
   NOT-VISIBLE FRAME-NAME                                               */
/* _RUN-TIME-ATTRIBUTES-END */
&ANALYZE-RESUME


/* Setting information for Queries and Browse Widgets fields            */

&ANALYZE-SUSPEND _QUERY-BLOCK FRAME fTreeViewParent
/* Query rebuild information for FRAME fTreeViewParent
     _Options          = ""
     _Query            is NOT OPENED
*/  /* FRAME fTreeViewParent */
&ANALYZE-RESUME

 


&ANALYZE-SUSPEND _UIB-CODE-BLOCK _CUSTOM _MAIN-BLOCK fFrameWin 


/* ***************************  Main Block  *************************** */

{amainsf1.i}

 &IF DEFINED(UIB_IS_RUNNING) <> 0 &THEN
   /* Now enable the interface  if in test mode - otherwise this happens when
      the object is explicitly initialized from its container. */
   RUN initializeObject.
&ENDIF

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME


/* **********************  Internal Procedures  *********************** */

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE applyEntry fFrameWin 
PROCEDURE applyEntry :
/*------------------------------------------------------------------------------
  Purpose:     Super Override
  Parameters:  
  Notes:       
------------------------------------------------------------------------------*/

  DEFINE INPUT PARAMETER pcField AS CHARACTER NO-UNDO.

  /* Code placed here will execute PRIOR to standard behavior. */

  /*RUN SUPER( INPUT pcField).*/
  /* APPLY "ENTRY" TO TreeView. */

  /* Code placed here will execute AFTER standard behavior.    */

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE dataAvailable fFrameWin 
PROCEDURE dataAvailable :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
    DEFINE INPUT PARAM pcRelative   AS CHAR NO-UNDO.
    
    DEFINE VARIABLE VhDataSource     AS HANDLE     NO-UNDO.
    DEFINE VARIABLE VcSurveyCode    AS CHARACTER  NO-UNDO.
    DEFINE VARIABLE VcRowIdent      AS CHAR       NO-UNDO.
    
    /* Only refresh if the tree has been initialized */
    IF NOT AlTreeInitialized THEN RETURN.

    {get DataSource VhDataSource}.

    IF pcRelative = "FIRST" THEN
    DO:
        SESSION:SET-WAIT-STATE("GENERAL").
                
        RUN PBuildTree ((LcCurrentRowIdent > "")). /* Reposition pending? */
    
        IF LcCurrentRowIdent > "" THEN
          {set AcSelectedNode LcCurrentRowIdent}.
        
        SESSION:SET-WAIT-STATE("").
    
    END.
    ELSE IF pcRelative = "DIFFERENT" THEN
    DO:
        {get rowIdent VcRowIdent VhDataSource}.
    
        IF VcRowIdent <> ? THEN
        DO:
            FIND ttNode WHERE ttNode.rowIdent = VcRowIdent NO-ERROR.

            IF AVAILABLE(ttNode)
            THEN RUN PTreeSelectNode(ttNode.id).
        END. 
    END.

    /*
    /* try this: */
    IF pcRelative = "FIRST" THEN
    DO:
        SESSION:SET-WAIT-STATE("GENERAL").
                
        RUN PBuildTree ((LcCurrentRowIdent > "")). /* Reposition pending? */
    
        SESSION:SET-WAIT-STATE("").

    END.
        
    {get DataSource VhDataSource}.
    {get rowIdent VcRowIdent VhDataSource}.
    
    IF VcRowIdent <> ? 
    THEN FIND ttNode WHERE ttNode.rowIdent = VcRowIdent NO-ERROR.

    IF AVAILABLE(ttNode)
    THEN RUN PTreeSelectNode(ttNode.id).
    */
    
    /* moving to a set with no info stuff */
    /* TODO:
    VcSurveyCode = {fnarg FcColValues 'fsurvey_code' VhDataSource}.
    IF VcSurveyCode = ? 
    THEN DO:
        PUBLISH "PEVENTUpdateLink":U(INPUT VhDataSource,"Survey").
        PUBLISH "PSurveyCode":U(INPUT "0",OUTPUT LhLevelData). 

        /* Blank out the question viewer when move to a set with nothing in it */
        PUBLISH "PEVENTTreeClick"(INPUT LcNodeKey,
                                  INPUT "No"
                                 ). 
    END.
    */

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE deleteComplete fFrameWin 
PROCEDURE deleteComplete :
/*------------------------------------------------------------------------------
  Purpose:     Run by ADM just after delete has completed
               Sets record to reposition to when query is reopened
------------------------------------------------------------------------------*/
  /* We want to reposition to a node near the one which has been deleted 
     after the query has reopened */  
     
  {get RowIdent LcCurrentRowIdent SOURCE-PROCEDURE} NO-ERROR.
  LcCurrentRowIdent = FcGetRowToPositionAfterDelete() NO-ERROR.
  {set AcRowIdentForReposition LcCurrentRowIdent SOURCE-PROCEDURE} NO-ERROR.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE disable_UI fFrameWin  _DEFAULT-DISABLE
PROCEDURE disable_UI :
/*------------------------------------------------------------------------------
  Purpose:     DISABLE the User Interface
  Parameters:  <none>
  Notes:       Here we clean-up the user-interface by deleting
               dynamic widgets we have created and/or hide 
               frames.  This procedure is usually called when
               we are ready to "clean-up" after running.
------------------------------------------------------------------------------*/
  /* Hide all frames. */
  HIDE FRAME fTreeViewParent.
  IF THIS-PROCEDURE:PERSISTENT THEN DELETE PROCEDURE THIS-PROCEDURE.
END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE enable_UI fFrameWin  _DEFAULT-ENABLE
PROCEDURE enable_UI :
/*------------------------------------------------------------------------------
  Purpose:     ENABLE the User Interface
  Parameters:  <none>
  Notes:       Here we display/view/enable the widgets in the
               user-interface.  In addition, OPEN all queries
               associated with each FRAME and BROWSE.
               These statements here are based on the "Other 
               Settings" section of the widget Property Sheets.
------------------------------------------------------------------------------*/
  {&OPEN-BROWSERS-IN-QUERY-fTreeViewParent}
END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE ExpandNode fFrameWin 
PROCEDURE ExpandNode :
/*------------------------------------------------------------------------------
  Purpose:     Expand/Collapse node given its key
  Notes:       Can pass in yes, no or ? to say whether to expand, collapse or
               toggle between expanded/collapsed
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAM i-cKey             AS CHAR     NO-UNDO.
  DEFINE INPUT PARAM i-logExpanded      AS LOG      NO-UNDO.
  
  /*
  DEF VAR chNode AS COM-HANDLE NO-UNDO.

  chNode = Key2ComHandle(i-cKey) NO-ERROR.

  IF i-logExpanded = ? 
  THEN chNode:expanded = NOT chNode:expanded.
  ELSE chNode:expanded = i-logExpanded.
  */

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE fetchDataSet fFrameWin 
PROCEDURE fetchDataSet :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
    DEFINE INPUT PARAMETER pcState      AS CHARACTER NO-UNDO.
    
    DEFINE VARIABLE VhDataSource        AS HANDLE  NO-UNDO.
    DEFINE VARIABLE hQuery              AS HANDLE  NO-UNDO.
    DEFINE VARIABLE ViList              AS INTEGER    NO-UNDO.
    
    /* If this is the case we we last on a survey */
    ViList = LOOKUP(LcNodeKey, LcIdList).
    IF ViList NE 0 
    THEN DO:
        {get DataSource VhDataSource}.
        IF VALID-HANDLE(VhDataSource) 
        THEN DO:
        
            {get DataHandle hQuery VhDataSource}.
            IF VALID-HANDLE(hQuery) THEN
            DO:
               hQuery:QUERY-OPEN() NO-ERROR.
            END.
        END.
    END.
    ELSE DO:
        IF VALID-HANDLE(LhLevelData) 
        THEN DO:
           
            {get DataHandle hQuery LhLevelData}.
            IF VALID-HANDLE(hQuery) THEN
            DO:
               hQuery:QUERY-OPEN() NO-ERROR.
            END.
        END.
    END.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE GetHitTestNode fFrameWin 
PROCEDURE GetHitTestNode :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
    DEFINE OUTPUT PARAMETER OchHandle       AS COM-HANDLE       NO-UNDO.
    
    /*
    DEFINE VARIABLE VmpViPoint              AS MEMPTR           NO-UNDO.  /* ViPoint FAR*  */
    DEFINE VARIABLE VmpViPointer            AS MEMPTR           NO-UNDO.
    DEFINE VARIABLE ViViPoint               AS INTEGER          NO-UNDO.
    DEFINE VARIABLE VdY                     AS DECIMAL          NO-UNDO.
    DEFINE VARIABLE VdX                     AS DECIMAL          NO-UNDO.
    
    SET-SIZE(VmpViPoint) = 8.
    
    /* get screen coordinates and convert to client area coordinates */
    RUN GetCursorPos(input VmpViPoint).
    RUN ScreenToClient(INPUT chTreeView:HWND, INPUT VmpViPoint).
    
    SET-SIZE(VmpViPointer) = 4.
    PUT-SHORT(VmpViPointer, 1) = GET-LONG(VmpViPoint,1) /*ABS( INTEGER(p-x) )*/.
    PUT-SHORT(VmpViPointer, 3) = GET-LONG(VmpViPoint,5) /*ABS( INTEGER(p-y) )*/.
    
    
    ASSIGN
      VdX = DECIMAL(GET-LONG(VmpViPoint,1)) * v-scale-x 
      VdY = DECIMAL(GET-LONG(VmpViPoint,5)) * v-scale-y 
    
      OchHandle= com-self:HitTest(VdX as float , VdY as float) 
      .
   */
END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE initializeObject fFrameWin 
PROCEDURE initializeObject :
/*------------------------------------------------------------------------------
  Purpose:     Super Override
  Parameters:  
  Notes:       
------------------------------------------------------------------------------*/
    DEFINE VARIABLE hDataSource AS HANDLE     NO-UNDO.

  /* Code placed here will execute PRIOR to standard behavior. */
  {get UIBMode l-cUIBMode}.
  
  /* Check for a data source SDO */
  RUN PCheckDataSource.

  RUN SUPER.

  SUBSCRIBE TO "PEVENTRemoveNode" ANYWHERE RUN-PROCEDURE "RemoveNode"  NO-ERROR.

 /* get container source for this object */
 {get ContainerSource LhContainerSource}.

  RUN resizeObject (FRAME {&FRAME-NAME}:height,FRAME {&FRAME-NAME}:WIDTH).
  
  /* At design time we disable the control. This allows developer to click 
     and select the tree SmartObject to resize it etc */
  IF l-cUIBMode = "DESIGN" THEN.
  /*
    ASSIGN
        TreeView:SENSITIVE = no.

  /* If design time then show default tree */
  IF l-cUIBMode = "DESIGN" THEN
  DO:
    RUN AddNode IN THIS-PROCEDURE (?,?,?,"DefaultDesignNode1","TreeView",?).
    RUN AddNode IN THIS-PROCEDURE (?,?,?,"DefaultDesignNode2","Control",?).
    RUN AddNode IN THIS-PROCEDURE (?,?,?,"DefaultDesignNode3","SmartObject",?).
  END.
 */   
    ELSE DO:
        RUN PBuildTree (FALSE). /* FALSE = No reposisition pending */
        AlTreeInitialized = TRUE.
    END.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PBuildTree fFrameWin 
PROCEDURE PBuildTree :
/*------------------------------------------------------------------------------
  Purpose:     Build tree from data-source object's query
------------------------------------------------------------------------------*/
    DEFINE INPUT PARAM plRepositionPending AS LOG NO-UNDO.

    DEFINE VARIABLE VcSurveyCode AS CHARACTER  NO-UNDO.

    PUBLISH "PEVENTTreeBuild":U(INPUT "START").

    /* Build the temp table */
    RUN PBuildTreeTable. 
    IF RETURN-VALUE > ""
    THEN RETURN RETURN-VALUE.
        
    /* ??? Not required for ftreefr0.w ???
    /* If including an unlinked root node then create it now */
    IF AlIncludeUnlinkedRoot THEN
    DO:
        RUN AddNode IN THIS-PROCEDURE
            (INPUT ? /* COM-HANDLE */,
             INPUT ?,
             INPUT ?,
             INPUT "UnlinkedRoot",
             INPUT AlIncludeUnlinkedRootText,
             INPUT ? 
             ).
    END.
    */

    /* Now build the tree */
    RUN PTreeCreateNodeFrame("").

    IF NOT plRepositionPending THEN
    DO:
        /* Now select first node in tree */
        IF AlAutoSelectRootNodeOnOpen THEN
            RUN PSelectFirstNode IN THIS-PROCEDURE. 
    END.
        
    /* ??? Not required for ftreefr0.w ???
    /* If auto expanding root then do it */
    IF AcAutoExpandSingleRoot = "YES" THEN
        chTreeRoot:expanded = yes.
    */

    PUBLISH "PEVENTTreeBuild":U(INPUT "END").

    RETURN "".

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PBuildTreeTable fFrameWin 
PROCEDURE PBuildTreeTable :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
    DEFINE VARIABLE hColumn                 AS HANDLE           NO-UNDO.
    DEFINE VARIABLE VcImageName             AS CHARACTER        NO-UNDO.

    DEFINE VARIABLE cId                     AS CHARACTER        NO-UNDO.
    DEFINE VARIABLE cText                   AS CHARACTER        NO-UNDO.
    DEFINE VARIABLE cparentId               AS CHARACTER        NO-UNDO.
    DEFINE VARIABLE cRowIdent               AS CHARACTER        NO-UNDO.
    DEFINE VARIABLE cLevelActive            AS CHARACTER        NO-UNDO.
    DEFINE VARIABLE cLevelLeaf              AS CHARACTER        NO-UNDO.

    DEFINE VARIABLE VhLevelQuery            AS HANDLE           NO-UNDO.
    DEFINE VARIABLE VhRowObject             AS HANDLE           NO-UNDO.

    DEFINE BUFFER ttNode FOR ttNode.

    EMPTY TEMP-TABLE ttNode.

    {get DataSource LhLevelData}.
    IF VALID-HANDLE(LhLevelData)
    THEN DO:
        {get DataHandle VhLevelQuery LhLevelData}.
        IF VALID-HANDLE(VhLevelQuery) THEN
        DO:
            IF NOT VhLevelQuery:IS-OPEN THEN
            RETURN "Query not open".

            VhRowObject = VhLevelQuery:GET-BUFFER-HANDLE(1).

            DO WHILE VhRowObject:AVAILABLE:

                ASSIGN
                    hColumn = VhRowObject:BUFFER-FIELD(AcIdFieldName)
                    cId = TRIM(hColumn:STRING-VALUE)

                    hColumn = VhRowObject:BUFFER-FIELD(AcTextFieldName)
                    cText = TRIM(hColumn:STRING-VALUE)

                    hColumn = VhRowObject:BUFFER-FIELD(AcparentIdFieldName)
                    cparentId = TRIM(hColumn:STRING-VALUE)
                    cparentId = "" WHEN cparentId = "?".

                /*
                hColumn = VhRowObject:BUFFER-FIELD(AcLevelLeaf)
                cLevelLeaf = hColumn:STRING-VALUE.
                */

                IF cLevelLeaf = "yes"
                THEN VcImageName = "LEAF".
                ELSE VcImageName = "LEVEL".

                {get RowIdent cRowIdent LhLevelData}.

                IF LcTreeMode <> "DEFN" AND cLevelActive = "no"
                THEN VhLevelQuery:GET-NEXT(NO-LOCK) NO-ERROR.
                ELSE DO:
                    /* Only build if you have a Parent there */
                    /*
                    IF  CAN-FIND(FIRST ttNode WHERE ttNode.id = cparentId USE-INDEX id)
                    THEN DO:
                    */

                         CREATE ttNode.
                         ASSIGN ttNode.id = cId  /* this is the level code field */
                                ttNode.nodeText = cText
                                ttNode.parentId = cparentId
                                ttNode.rowident = cRowIdent
                                ttNode.ImageName = VcImageName
                                ttNode.active = "yes".
                    /*
                    END.
                    */
                    VhLevelQuery:GET-NEXT(NO-LOCK) NO-ERROR.
                END.
            END. /* RowObj Avail */
        END. /* valid query */

    END. /* valid data */
    ELSE RETURN "No data source".

    DEFINE BUFFER xyz FOR ttNode.
    FIND xyz where xyz.id = "1" NO-ERROR.

    RETURN "".
END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PCheckDataSource fFrameWin 
PROCEDURE PCheckDataSource :
/*------------------------------------------------------------------------------
  Purpose:     Check for a data source SDO

               If its there, go into 'SDB mode' (SmartDataBrowser)
               Also ensure that the SDO will reopen the query after an add
               modify and delete
------------------------------------------------------------------------------*/
    DEFINE VARIABLE VhDataSource                AS HANDLE NO-UNDO.

    {get dataSource VhDataSource}.
    
    IF VALID-HANDLE(VhDataSource) THEN
    DO:
        LlSDBMode = yes.
   
       /* Configure dataSource so query is reopened after add,modify and delete */
       RUN modifyListProperty
            (INPUT VhDataSource,
             INPUT "ADD",
             INPUT "AcReopenQueryAfterButtons",
             INPUT 'add,modify,addchild').  /* Cannot have delete in here, otherwise 
                                               the node gets re-added, before the 
                                               batch program has had chance to delete
                                              */ 
    END. 
   
END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PConvImageNameToNumber fFrameWin 
PROCEDURE PConvImageNameToNumber :
/*------------------------------------------------------------------------------
  Purpose:     Convert the name of an image to it's number (index) in the
               imageview OCX
------------------------------------------------------------------------------*/

    DEFINE INPUT  PARAM IcImageName              AS CHAR             NO-UNDO.
    DEFINE OUTPUT PARAM OiImageNumber            AS INTEGER          NO-UNDO.

   CASE IcImageName:
        WHEN "SURVEY"           THEN OiImageNumber = 1.
        WHEN "LEVEL"            THEN OiImageNumber = 4.
        WHEN "LEAF"             THEN OiImageNumber = 7.

        OTHERWISE OiImageNumber = 1.  /* Survey is default */
        
    END CASE. 

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PNodeSelected fFrameWin 
PROCEDURE PNodeSelected :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
    DEFINE INPUT PARAM IcNodeId AS CHARACTER NO-UNDO.

    LcLevelCode = IcNodeId.
    
    IF LlSDBMode 
    THEN DO:
        FIND ttNode WHERE ttNode.id = IcNodeId.
        {set AcSelectedNode ttNode.rowIdent}.
    END.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PSelectFirstNode fFrameWin 
PROCEDURE PSelectFirstNode :
/*------------------------------------------------------------------------------
  Purpose:     
------------------------------------------------------------------------------*/
    DEFINE VARIABLE ViList          AS INTEGER    NO-UNDO.
    DEFINE VARIABLE hDataSource     AS HANDLE     NO-UNDO.
    
    /* only try to reposition when in the same set otherwise problems */
    {get DataSource hDataSource}.
    
    LcCurrentSet = DYNAMIC-FUNCTION('FcCurrentSet' IN LhContainerSource).
    
    IF LcCurrentSet = LcOldSet AND LENGTH(LcNodeKey) > 0
    THEN FIND ttNode WHERE ttNode.rowIdent = LcNodeKey. /* Select the current node where possible */
    ELSE FIND FIRST ttNode WHERE ttNode.parentId = "" NO-ERROR. /* Select first one */ 

    IF AVAILABLE(ttNode) 
    THEN DO:
        RUN PTreeSelectNode(ttNode.id).
        RUN PTreeExpandNode(ttNode.id). /* Ensure it's expanded too (if it's a parent) ??? possibly take out */
    END.

    LcOldSet = LcCurrentSet.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PSwapImageForNode fFrameWin 
PROCEDURE PSwapImageForNode :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
    DEFINE INPUT PARAMETER  IcNodeKey           AS CHAR     NO-UNDO.
    DEFINE INPUT PARAMETER IiNewImage           AS INTEGER  NO-UNDO.

    /*
    DEF VAR chNode AS COM-HANDLE NO-UNDO.

    chNode = Key2ComHandle(IcNodeKey) NO-ERROR.
    
    chNode:SelectedImage = IiNewImage NO-ERROR.
    */

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PSwapTextForNode fFrameWin 
PROCEDURE PSwapTextForNode :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
    DEFINE INPUT PARAMETER  IcNodeKey           AS CHAR     NO-UNDO.
    DEFINE INPUT PARAMETER  IcNewText           AS CHAR     NO-UNDO.

    /*
    DEF VAR chNode AS COM-HANDLE NO-UNDO.
  
    chNode = Key2ComHandle(IcNodeKey) NO-ERROR.
    chNode:Text = IcNewText NO-ERROR.
    */

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PTransactionValid fFrameWin 
PROCEDURE PTransactionValid :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
    DEFINE INPUT PARAM lAddedRecord     AS LOG          NO-UNDO.
    DEFINE INPUT PARAM lDeletedRecord   AS LOG          NO-UNDO.
    DEFINE INPUT PARAM lModifiedRecord  AS LOG          NO-UNDO.

    LlTransactionCompleted = yes.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PTreeCollapse fFrameWin 
PROCEDURE PTreeCollapse :
/*------------------------------------------------------------------------------
  Purpose:     run any published tree collapse procedures
  Parameters:  handle of selected node
  Notes:       
------------------------------------------------------------------------------*/
    DEFINE INPUT PARAM chNode AS COM-HANDLE NO-UNDO.
/*  TODO
    PUBLISH "PTreeCollapse"(INPUT chNode:Key,            /* key for clicked item */
                            INPUT chNode:selected        /* is the node to be collapsed currently selected */
                            ).
*/
END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PTreeDblClick fFrameWin 
PROCEDURE PTreeDblClick :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
    DEFINE INPUT PARAM chNode           AS COM-HANDLE   NO-UNDO.

    /* TODO
    PUBLISH "PTreeDblClick" (INPUT chNode:Key,             /* key for clicked item */
                             INPUT (chNode:Children = 0)   /* is item a leaf node? */).
     */
END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE PTreeExpand fFrameWin 
PROCEDURE PTreeExpand :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
    DEFINE INPUT PARAM chNode AS COM-HANDLE NO-UNDO.

    DEFINE VARIABLE VlOpenQuery AS LOGICAL    NO-UNDO.
    /* TODO 
    PUBLISH "PTreeExpand" (INPUT chNode:KEY). /* key for clicked item */
    */
END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE RemoveNode fFrameWin 
PROCEDURE RemoveNode :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAM IcKey              AS CHAR         NO-UNDO.

  DEFINE VARIABLE chNode                AS COM-HANDLE   NO-UNDO.
  
  /* TODO
  chNode = chTreeView:Nodes:Remove(IcKey) NO-ERROR. 
  */
  MESSAGE "Refresh screen to update treeview." VIEW-AS ALERT-BOX INFORMATION.

  ASSIGN LcNodeKey = ?.
 
END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE resizeObject fFrameWin 
PROCEDURE resizeObject :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
 DEFINE INPUT PARAMETER p-height AS DECIMAL NO-UNDO.
  DEFINE INPUT PARAMETER p-width  AS DECIMAL NO-UNDO.

  ASSIGN
      FRAME {&FRAME-NAME}:HEIGHT = p-height
      FRAME {&FRAME-NAME}:WIDTH  = p-width
      FRAME {&FRAME-NAME}:VIRTUAL-HEIGHT-PIXELS = FRAME {&FRAME-NAME}:HEIGHT-PIXELS
      FRAME {&FRAME-NAME}:VIRTUAL-WIDTH-PIXELS  = FRAME {&FRAME-NAME}:WIDTH-PIXELS

  NO-ERROR.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

/* ************************  Function Implementations ***************** */

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION FcGetRowToPositionAfterDelete fFrameWin 
FUNCTION FcGetRowToPositionAfterDelete RETURNS CHARACTER
  ( /* parameter-definitions */ ) :
/*------------------------------------------------------------------------------
  Purpose:  Determine which row to position to after the selected row is
            deleted (e.g. its next sibling or its parent)
            Returns the node's key
------------------------------------------------------------------------------*/

  /* TODO
  /* Is there a next sibling? */
  IF chTreeView:SelectedItem:Next > 0 THEN
    RETURN chTreeView:SelectedItem:Next:Key.
  /* Is there a prev sibling? */
  ELSE IF chTreeView:SelectedItem:Previous > 0 THEN
    RETURN chTreeView:SelectedItem:Previous:Key.
  /* Is there a parent? */
  ELSE IF chTreeView:SelectedItem:Parent > 0 THEN
    RETURN chTreeView:SelectedItem:Parent:Key.
  ELSE
    RETURN "".
  */
END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION FcNodeImageName fFrameWin 
FUNCTION FcNodeImageName RETURNS CHARACTER
  ( BUFFER ttNode FOR ttNode ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  
------------------------------------------------------------------------------*/

    IF   ttNode.ACTIVE = "no"
    THEN RETURN "inactive".
    ELSE RETURN ttNode.imageName.

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION FlIsNodeExpanded fFrameWin 
FUNCTION FlIsNodeExpanded RETURNS LOGICAL
  ( INPUT IcNodeKey         AS CHAR ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  
------------------------------------------------------------------------------*/

  /* TODO
  DEF VAR chNode AS COM-HANDLE NO-UNDO.

  chNode = Key2ComHandle(IcNodeKey).
  RETURN chNode:expanded.
  */

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION GetAcSelectedItemKey fFrameWin 
FUNCTION GetAcSelectedItemKey RETURNS CHARACTER
  ( /* parameter-definitions */ ) :
/*------------------------------------------------------------------------------
  Purpose:  Use to get key of currently selected node
    Notes:  {get AcSelectedItemKey variabletoreceivevalue h_treeview}.
------------------------------------------------------------------------------*/
  /*
  IF chTreeView:SelectedItem = 0 OR chTreeView:SelectedItem = ? THEN
      RETURN ?.

  RETURN chTreeView:SelectedItem:KEY.   /* Function return value. */
  */

   FIND ttNode WHERE ttNode.id = LhTreeSelectedText:NAME NO-ERROR.
   IF AVAILABLE(ttNode) 
   THEN RETURN ttNode.rowIdent.
   
END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION getTreeInitialized fFrameWin 
FUNCTION getTreeInitialized RETURNS CHARACTER
  ( /* parameter-definitions */ ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  
------------------------------------------------------------------------------*/

  RETURN STRING(AlTreeInitialized,"Yes/No").   /* Function return value. */

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION Key2ComHandle fFrameWin 
FUNCTION Key2ComHandle RETURNS COM-HANDLE
  ( INPUT i-cKey AS CHAR ) :
/*------------------------------------------------------------------------------
  Purpose:  Convert key (char) to component handle
    Notes:  temp table TTAddedNodes is used to keep a record of which nodes have
            already been added.  The old method was to do 'chReturn = chTreeView:Nodes:Item(i-cKey)'
            with NO-ERROR so that the ocx didn't raise an error BUT this caused a memory leak problem
            each time it is used
------------------------------------------------------------------------------*/
    
    /* TODO: scrap ?
    DEFINE VARIABLE chReturn    AS COM-HANDLE   NO-UNDO.

    IF CAN-FIND(FIRST TTAddedNodes WHERE TTAddedNodes.TTKey = i-cKey NO-LOCK) 
    THEN chReturn = chTreeView:Nodes:Item(i-cKey) NO-ERROR.
    ELSE chReturn = ?.
    
    RETURN chReturn.
    */ 

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION SetAcAutoExpandSingleRoot fFrameWin 
FUNCTION SetAcAutoExpandSingleRoot RETURNS CHARACTER
  ( INPUT i AS CHAR /* parameter-definitions */ ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  Set to True by the Frame the Tree is in, this 
            will allow a parent node to auotmatically expand 
            upon build
------------------------------------------------------------------------------*/

  AcAutoExpandSingleRoot = i.
  
END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION SetAcIdFieldName fFrameWin 
FUNCTION SetAcIdFieldName RETURNS CHARACTER
  ( INPUT i AS CHAR ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  Set by Container Frame
------------------------------------------------------------------------------*/

    AcIdFieldName = i.

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION setAcImageListToUse fFrameWin 
FUNCTION setAcImageListToUse RETURNS CHARACTER
  ( INPUT Ic AS CHAR ) :
/*------------------------------------------------------------------------------
  Purpose:  Set which imagelist to use
            NORMAL - normal image list (16x16 with explorer-like folders)
            EXPRESSIONS - custom size image list with special images for expressions
    Notes:  
------------------------------------------------------------------------------*/

/* TODO 
    
  CASE Ic:
    WHEN "NORMAL"       THEN chTreeView:ImageList   = chImageListNormal.
    OTHERWISE           MESSAGE SUBST("AcImageListToUse called with invalid image list: &1",Ic).
  END CASE.
*/
  RETURN "".

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION SetAcLevelActive fFrameWin 
FUNCTION SetAcLevelActive RETURNS CHARACTER
  ( INPUT i AS CHAR ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  Set by Container Frame
------------------------------------------------------------------------------*/

    AcLevelActive = i.   /* Function return value. */

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION SetAcLevelHandle fFrameWin 
FUNCTION SetAcLevelHandle RETURNS HANDLE
  ( INPUT i AS HANDLE ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  
------------------------------------------------------------------------------*/

  AcLevelHandle = i.   /* Function return value. */

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION SetAcLevelLeaf fFrameWin 
FUNCTION SetAcLevelLeaf RETURNS CHARACTER
  ( INPUT i AS CHAR ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  Set by Container Frame
------------------------------------------------------------------------------*/

  AcLevelLeaf = i.   /* Function return value. */

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION SetAcparentIdFieldName fFrameWin 
FUNCTION SetAcparentIdFieldName RETURNS CHARACTER
  ( INPUT i AS CHAR ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  Set by Container Frame
------------------------------------------------------------------------------*/

    AcparentIdFieldName = i.

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION SetAcSelectedNode fFrameWin 
FUNCTION SetAcSelectedNode RETURNS CHARACTER
  ( INPUT cRowIdent AS CHAR ) :  
/*------------------------------------------------------------------------------
  Purpose:  Setting this attribute will just fire fetchRowIdent in the data
            source SDO
------------------------------------------------------------------------------*/  
  DEFINE VARIABLE hDataSource    AS HANDLE        NO-UNDO.
  DEFINE VARIABLE hQuery         AS HANDLE     NO-UNDO.
  DEFINE VARIABLE VcLeaf         AS CHARACTER  NO-UNDO.
  DEFINE VARIABLE VlOpen         AS LOGICAL    NO-UNDO.

    /* Has a survey been selected? */
    IF LOOKUP(cRowIdent, LcIdList) NE 0 /* LcIdList is built in the tree build, and is a comma separated list of the cRowidents for Surveys */
    THEN DO:
        {get DataSource hDataSource}.
        
        /* the Container sets a link to the relevant SDO */
        PUBLISH "PEVENTUpdateLink":U(INPUT hDataSource,"Survey").

        IF VALID-HANDLE(hDataSource)
        THEN DYNAMIC-FUNCTION("fetchRowIdent" IN hDataSource,cRowIdent,"RowIdent") NO-ERROR. /* repositions to the desired row in the SDO */
        
        IF LcNodeKey <> ?
        THEN LcNodeKey = cRowIdent. 
        /* otherwise On Initial build the ques hasn't a handle to 
           the panel to disable.(Could check for Valid-handle there
           but MED has locked */

        IF LENGTH(LcOldSet) > 0 THEN /* !!! Bug - OldSet only reset when reopened  */
        PUBLISH "PEVENTTreeClick"(INPUT LcNodeKey,
                                  INPUT "No"
                                  ). 
        RETURN "".  
  END.
  ELSE DO:
        /* LhLevelData is set Globally in the Build Tree for the level SDO */
        IF VALID-HANDLE(LhLevelData) THEN
        DO:
            PUBLISH "PEVENTUpdateLink":U(INPUT LhLevelData,"Level"). 

            DYNAMIC-FUNCTION('assignQuerySelection':U IN LhLevelData, INPUT "flevel_code",
                                                                      INPUT LcLevelCode,                      
                                                                      INPUT "="
                                                                      ).
            /* TODO: this causes the data to be reloaded and the tree being rebuilt
            DYNAMIC-FUNCTION('openQuery':U IN LhLevelData).
            */

            /* In this Frame it is refreshing the Browse Twice, 
               so Block it fetching it in this instance */
            IF STRING(LhContainerSource:FILE-NAME) = "fdefnfr0.w" THEN 
            DYNAMIC-FUNCTION("fetchRowIdent" IN LhLevelData,cRowIdent,"RowIdent") NO-ERROR.

            ASSIGN LcNodeKey = cRowIdent.

            /* TODO: leaf is not used
            VcLeaf = {fnarg FcColValues 'leaf' LhLevelData}.
            PUBLISH "PEVENTTreeClick"(INPUT LcNodeKey,
                                      INPUT VcLeaf
                                      ).
            */
            PUBLISH "PEVENTTreeClick"(INPUT LcNodeKey).
        END.
        RETURN "".   
  END. 
 
END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION SetAcSurveyActive fFrameWin 
FUNCTION SetAcSurveyActive RETURNS CHAR
  ( INPUT i AS CHAR ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes: Set by Container Frame 
------------------------------------------------------------------------------*/

    AcSurveyActive = i.   /* Function return value. */

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION SetAcSurveyCode fFrameWin 
FUNCTION SetAcSurveyCode RETURNS CHARACTER
  ( INPUT i AS CHAR) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  Set by Container Frame
------------------------------------------------------------------------------*/

    AcSurveyCode = i.   /* Function return value. */

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION SetAcSurveyDesc fFrameWin 
FUNCTION SetAcSurveyDesc RETURNS CHARACTER
  ( INPUT i AS CHAR ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  Set by Container Frame
------------------------------------------------------------------------------*/

    AcSurveyDesc = i.   /* Function return value. */

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION SetAcTextFieldName fFrameWin 
FUNCTION SetAcTextFieldName RETURNS CHARACTER
  ( INPUT i AS CHAR ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  Set by Container Frame
------------------------------------------------------------------------------*/

    AcTextFieldName = i.

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION SetAcTreeMode fFrameWin 
FUNCTION SetAcTreeMode RETURNS CHARACTER
  ( INPUT i AS CHAR ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes: Set by Container Frame 
------------------------------------------------------------------------------*/

    LcTreeMode = i.   /* Function return value. */

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION SetAiImage fFrameWin 
FUNCTION SetAiImage RETURNS CHARACTER
  ( INPUT val AS CHAR /* parameter-definitions */ ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  
------------------------------------------------------------------------------*/

  AiImage = INT(val).

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION setAiLeafImage fFrameWin 
FUNCTION setAiLeafImage RETURNS CHARACTER
  ( INPUT Ii AS INT ) :
/*------------------------------------------------------------------------------
  Purpose:  Set the image number to be used for the image for leaf nodes
------------------------------------------------------------------------------*/

  AiLeafImage = Ii.

  RETURN "".   /* Function return value. */

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION setAiLeafSelImage fFrameWin 
FUNCTION setAiLeafSelImage RETURNS CHARACTER
  ( INPUT Ii AS INT ) :
/*------------------------------------------------------------------------------
  Purpose:  Set the image number to be used for the selected image for leaf nodes
------------------------------------------------------------------------------*/

  AiLeafSelImage = Ii.

  RETURN "".   /* Function return value. */

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION setAiSelectedImage fFrameWin 
FUNCTION setAiSelectedImage RETURNS CHARACTER
  ( INPUT Ii AS INT ) :
/*------------------------------------------------------------------------------
  Purpose:  Sets the image index used for the "selected image" of each node
            This is the image shown for the selected node
    Notes:  
------------------------------------------------------------------------------*/

  AiSelectedImage = Ii.

  RETURN "".   /* Function return value. */

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION setAlAutoExpandAllNodes fFrameWin 
FUNCTION setAlAutoExpandAllNodes RETURNS CHARACTER
  ( INPUT Il AS LOG ) :
/*------------------------------------------------------------------------------
  Purpose:  Set this attr to determine whether the tree will set the expanded
            property to 'yes' on every new node
    Notes:  
------------------------------------------------------------------------------*/

  AlAutoExpandAllNodes = Il.

  RETURN "".   /* Function return value. */

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION setAlAutoSelectRootNodeOnOpen fFrameWin 
FUNCTION setAlAutoSelectRootNodeOnOpen RETURNS CHARACTER
  ( INPUT Il AS LOG ) :
/*------------------------------------------------------------------------------
  Purpose:  If this is true then root node will be selected when tree opens
------------------------------------------------------------------------------*/

  AlAutoSelectRootNodeOnOpen = Il.

  RETURN "".   /* Function return value. */

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION setAlAutoSortAllNodes fFrameWin 
FUNCTION setAlAutoSortAllNodes RETURNS CHARACTER
  ( INPUT Il AS LOG /* parameter-definitions */ ) :
/*------------------------------------------------------------------------------
  Purpose:  Whether to auto sort each new node added
    Notes:  
------------------------------------------------------------------------------*/

  /* TODO
  AlAutoSortAllNodes = Il.

  /* Set the whole treeview's sorted property also. This affects the sorting of nodes
     added as root nodes
  */
  IF chTreeView > 0 THEN
      chTreeView:Sorted = Il.
  */
  RETURN "".   /* Function return value. */

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION setAlIncludeUnlinkedRoot fFrameWin 
FUNCTION setAlIncludeUnlinkedRoot RETURNS CHARACTER
  ( INPUT Il AS LOG ) :
/*------------------------------------------------------------------------------
  Purpose:  Determines whether SDB tree includes a root node which is not linked
            to a record in the SDO
    Notes:  
------------------------------------------------------------------------------*/

  AlIncludeUnlinkedRoot = Il.
  
  RETURN "".   /* Function return value. */

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION setAlIncludeUnlinkedRootText fFrameWin 
FUNCTION setAlIncludeUnlinkedRootText RETURNS CHARACTER
  ( INPUT Ic AS CHAR ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  
------------------------------------------------------------------------------*/

  AlIncludeUnlinkedRootText = Ic.

  RETURN "".   /* Function return value. */

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION setAlSecurityCheck fFrameWin 
FUNCTION setAlSecurityCheck RETURNS CHARACTER
  ( INPUT Il    AS LOGICAL ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  
------------------------------------------------------------------------------*/

    AlSecurityCheck = Il.

  RETURN "".   /* Function return value. */

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _FUNCTION SetAlTreeInitialized fFrameWin 
FUNCTION SetAlTreeInitialized RETURNS CHARACTER
  ( INPUT i AS CHAR ) :
/*------------------------------------------------------------------------------
  Purpose:  
    Notes:  
------------------------------------------------------------------------------*/

    AlTreeInitialized = (i = "YES").

END FUNCTION.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

